import 'package:example/localization/labels.dart';
import 'package:example/ui/activity_data_insert.dart';
import 'package:example/ui/activity_private.dart';
import 'package:example/ui/bottom_sheet_data_insert.dart';
import 'package:flutter/material.dart' hide DialogRoute;
import 'package:flutterx_application/flutterx_application.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

class HomeActivity extends StatefulWidget {
  static final ActivityRoute<void> route =
      ActivityRoute.builder(HomeActivity, builder: (context, args) => const HomeActivity._());

  const HomeActivity._({Key? key}) : super(key: key);

  @override
  State<HomeActivity> createState() => _HomeActivityState();
}

const Widget divider = Divider(indent: 32, endIndent: 32, height: 64);

class _HomeActivityState extends State<HomeActivity> {
  int _counter = 0;
  String? _navResult;

  @override
  Widget build(BuildContext context) => Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: AppBar(title: Text(labels.appName)),
      body: Center(
        child: Column(mainAxisAlignment: MainAxisAlignment.center, children: [
          Text(labels.contentText(_counter)),
          divider,
          ..._localizationDemo(context),
          divider,
          ..._navigationDemo(context),
          divider,
          ElevatedButton(
              onPressed: () => PrivateActivity.open(context, data: 'hello'), child: const Text('Private page')),
        ]),
      ),
      floatingActionButton:
          FloatingActionButton(onPressed: () => setState(() => _counter++), child: const Icon(Icons.add)));

  Iterable<Widget> _localizationDemo(BuildContext context) sync* {
    final app = Application.require(context);
    final supportedLocales = app.supportedLocales;
    final defaultLocale = app.defaultLocale;
    final currentLocale = Localizations.localeOf(context);
    yield Text(labels.chooseLanguage);
    yield const SizedBox(height: 8);
    yield ToggleButtons(
        isSelected: supportedLocales.map((locale) => locale == currentLocale).toList(growable: false),
        onPressed: (value) => app.enforcedLocale = supportedLocales[value],
        textStyle: Theme.of(context).textTheme.headline6,
        borderRadius: BorderRadius.circular(4),
        borderWidth: 1,
        borderColor: Colors.grey[600],
        selectedBorderColor: Colors.grey[900],
        children: supportedLocales
            .map((locale) => Text((locale == defaultLocale ? '*' : '') + locale.languageCode.toUpperCase()))
            .toList(growable: false));
  }

  static final DialogRoute<String?> _dialogRoute = DialogRoute.alertBuilder('alert',
      onDismiss: (context, outArgs) => 'DISMISSED (text clicked: ${outArgs['clicked'] == true})',
      positiveAction: AlertDialogAction(
          onAction: (context, outArgs) => 'POSITIVE PRESSED (text clicked: ${outArgs['clicked'] == true})'),
      negativeAction: AlertDialogAction(
          onAction: (context, outArgs) => 'NEGATIVE PRESSED (text clicked: ${outArgs['clicked'] == true})'),
      neutralAction: AlertDialogAction(
          onAction: (context, outArgs) => 'NEUTRAL PRESSED (text clicked: ${outArgs['clicked'] == true})'),
      builder: (context, args, outArgs) {
        final key = GlobalKey();
        return MaterialStatefulWidget(
            onPressed: () => outArgs['clicked'] = true,
            builder: (context, states, child) {
              final content = Material(
                key: key,
                elevation: states.pressed || states.hovered || states.dragged ? 10 : 2,
                color: Colors.orange[100],
                child: SizedBox(
                  height: 100,
                  width: 240,
                  child: RichText(
                      text: TextSpan(
                          text: 'CLICK HERE',
                          style: Theme.of(context).textTheme.bodyText2!.copyWith(
                              decoration: states.pressed || states.hovered || states.dragged
                                  ? TextDecoration.underline
                                  : null))),
                ),
              );
              return LongPressDraggable(
                  feedback: content,
                  childWhenDragging: const SizedBox(height: 100, width: 240),
                  onDragStarted: () => states.dragged = true,
                  onDragEnd: (_) => states.dragged = false,
                  child: content);
            });
      });

  Iterable<Widget> _navigationDemo(BuildContext context) sync* {
    yield Text(labels.navigation);
    yield const SizedBox(height: 8);
    yield Padding(
        padding: const EdgeInsets.symmetric(horizontal: 16),
        child: Wrap(
            alignment: WrapAlignment.center,
            children: <Widget>[
              ElevatedButton(
                  onPressed: () => DataInsertActivity.open(context, initialValue: _navResult).then(_onNavResult),
                  child: Text(labels.openActivity)),
              ElevatedButton(
                  onPressed: () => _dialogRoute.open(context).then(_onNavResult), child: Text(labels.openDialog)),
              ElevatedButton(
                  onPressed: () => DataInsertBottomSheet.open(context, initialValue: _navResult).then(_onNavResult),
                  child: Text(labels.openBottomSheet)),
            ].interpolate(const SizedBox(width: 16)).toList(growable: false)));
    yield const SizedBox(height: 8);
    yield Text(labels.navigationResult(_navResult.toString()));
  }

  void _onNavResult(String? result) => mounted ? setState(() => _navResult = result) : null;
}
