part of 'application.dart';

class _AppBinding extends WidgetsBindingObserver {
  final Map<LifecycleObserver, AppLifecycleState> _observers = HashMap();
  final _ApplicationImplState app;

  _AppBinding(this.app) {
    WidgetsBinding.instance!.addObserver(this);
  }

  @override
  void didChangeMetrics() {
    super.didChangeMetrics();
    app.isInputMethodVisible.value = WidgetsBinding.instance!.window.viewInsets.bottom > 0;
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    for (final observer in _observers.entries)
      if (observer.value != state) {
        _observers[observer.key] = state;
        observer.key(state);
      }
  }

  void addLifecycleObserver(LifecycleObserver observer) {
    if (_observers.containsKey(observer)) return;
    final state = WidgetsBinding.instance!.lifecycleState!;
    _observers[observer] = state;
    observer(state);
  }

  void removeLifecycleObserver(LifecycleObserver observer) => _observers.remove(observer);

  void dispose() => WidgetsBinding.instance!.removeObserver(this);
}

class _FrameCallbackLiveEvent extends LiveEvent<Duration> {
  static final _FrameCallbackLiveEvent instance = _FrameCallbackLiveEvent._();
  bool _registered = false;

  _FrameCallbackLiveEvent._() : super(dispatcher: Dispatcher.immediate);

  @override
  bool shouldUpdateValue(Duration oldValue, Duration newValue) => isActive;

  @override
  void onActive() {
    super.onActive();
    if (_registered) return;
    WidgetsBinding.instance!.addPersistentFrameCallback(setData);
    _registered = true;
  }
}
