part of 'application.dart';

class _ApplicationImpl extends StatefulWidget {
  final String name;
  final String initialRoute;
  final Set<LabelInterface>? labels;
  final ValueSetter<LabelInterface>? onLabel;
  final MaterialAppFactory materialAppFactory;

  const _ApplicationImpl(
      {Key? key,
      required this.name,
      required this.initialRoute,
      this.labels,
      this.onLabel,
      required this.materialAppFactory})
      : assert(labels == null || onLabel != null, 'onLabel must be provided along with labels'),
        super(key: key);

  @override
  _ApplicationImplState createState() => _ApplicationImplState();
}

class _ApplicationImplState extends State<_ApplicationImpl> implements Application {
  @override
  final MutableLiveData<bool> isInputMethodVisible = MutableLiveData(initialValue: false);
  @override
  final LiveEvent<Duration> frameCallback = _FrameCallbackLiveEvent.instance;

  @override
  List<Locale> supportedLocales = const [];

  @override
  late Locale defaultLocale;

  @override
  Locale? get enforcedLocale => _enforcedLocale;

  @override
  set enforcedLocale(Locale? value) => setState(() => _enforcedLocale = value);

  late _AppBinding _binding;
  GlobalKey<State<MaterialApp>> _key = GlobalKey();
  Locale? _enforcedLocale;

  @override
  void initState() {
    super.initState();
    _binding = _AppBinding(this);
  }

  @override
  void dispose() {
    _binding.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final initialRoute = widget.initialRoute;
    final title = widget.name;
    final labelInterfaceDelegate = widget.labels?.let((labels) => LabelInterfaceDelegate(labels, widget.onLabel!));
    final localizationsDelegates = <LocalizationsDelegate>[
      if (labelInterfaceDelegate != null) labelInterfaceDelegate,
      GlobalMaterialLocalizations.delegate,
      GlobalWidgetsLocalizations.delegate,
      GlobalCupertinoLocalizations.delegate,
    ];
    final systemLocales = WidgetsBinding.instance!.window.locales;
    supportedLocales = widget.labels?.map((label) => label.locale).toList(growable: false) ?? systemLocales;
    defaultLocale = basicLocaleListResolution(systemLocales, supportedLocales);
    final root = widget.materialAppFactory(
        _key, initialRoute, AppRoute.onGenerateRoute, title, _enforcedLocale, localizationsDelegates, supportedLocales);
    assert(() {
      WidgetsBinding.instance!.addPostFrameCallback((_) {
        final app = _key.currentWidget;
        if (app is! MaterialApp) throw StateError('MaterialAppFactory must set key parameter to the MaterialApp');
        if (app.initialRoute != initialRoute ||
            app.onGenerateRoute != AppRoute.onGenerateRoute ||
            app.title != title ||
            (labelInterfaceDelegate != null && app.localizationsDelegates?.contains(labelInterfaceDelegate) != true) ||
            (widget.labels != null && !app.supportedLocales.toSet().containsAll(supportedLocales)))
          throw StateError('MaterialAppFactory must set all given parameters');
      });
      return true;
    }());
    return root;
  }

  @override
  void restart() => setState(() => _key = GlobalKey());

  @override
  void exit() => SystemNavigator.pop(animated: true).then((value) => io.Platform.isIOS ? io.exit(0) : null);

  @override
  void addLifecycleObserver(LifecycleObserver observer) => _binding.addLifecycleObserver(observer);

  @override
  void removeLifecycleObserver(LifecycleObserver observer) => _binding.removeLifecycleObserver(observer);
}

MaterialApp _defaultMaterialAppFactory(
  GlobalKey key,
  String initialRoute,
  RouteFactory onGenerateRoute,
  String title,
  Locale? locale,
  Iterable<LocalizationsDelegate> localizationsDelegates,
  Iterable<Locale> supportedLocales,
) =>
    MaterialApp(
        key: key,
        initialRoute: initialRoute,
        onGenerateRoute: onGenerateRoute,
        title: title,
        locale: locale,
        localizationsDelegates: localizationsDelegates,
        supportedLocales: supportedLocales,
        debugShowCheckedModeBanner: false);
