import 'package:flutter/material.dart';
import 'package:flutterx_application/flutterx_application.dart';

/// Represents a simple Line
///
/// [direction] axis of the line. Defaults to [Axis.horizontal]
/// [color] of the line
/// [length] of the line
/// [thickness] of the line
/// [margin] of the line
class Line extends StatelessWidget {
  final Axis direction;
  final Color? color;
  final double length;
  final double thickness;
  final EdgeInsetsGeometry? margin;

  const Line({
    Key? key,
    this.direction = Axis.horizontal,
    this.color,
    this.length = double.infinity,
    this.thickness = 1,
    this.margin,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) => wrapIfNotNull<Widget, EdgeInsetsGeometry>(
      item: margin,
      wrap: (child, margin) => Padding(padding: margin, child: child),
      child: CustomPaint(
          size: direction == Axis.horizontal ? Size(length, thickness) : Size(thickness, length),
          painter: _LinePainter(
              direction, color ?? DividerTheme.of(context).color ?? Theme.of(context).dividerColor, thickness)));
}

class _LinePainter extends CustomPainter {
  final Axis direction;
  final Paint _paint;

  _LinePainter(this.direction, Color color, double thickness)
      : _paint = Paint()
          ..color = color
          ..strokeWidth = thickness;

  @override
  void paint(Canvas canvas, Size size) => direction == Axis.horizontal
      ? canvas.drawLine(Offset(0, size.height / 2), Offset(size.width, size.height / 2), _paint)
      : canvas.drawLine(Offset(size.width / 2, 0), Offset(size.width / 2, size.height), _paint);

  @override
  bool shouldRepaint(_LinePainter oldDelegate) =>
      oldDelegate.direction != direction ||
      oldDelegate._paint.color != _paint.color ||
      oldDelegate._paint.strokeWidth != _paint.strokeWidth;
}
